<?php
/**
 * custom_field_admin.php
 *
 * Custom field management
 *
 * @author          Opencart-api.com
 * @copyright       2022
 * @license         License.txt
 * @version         1.0
 * @link            https://opencart-api.com/product/opencart-rest-admin-api/
 * @documentations  https://opencart-api.com/opencart-rest-api-documentations/
 */
namespace Opencart\Catalog\Controller\Extension\Restadmin\Rest;

use \Opencart\Extension\Restadmin\System\Engine\RestAdminController;
use \Opencart\System\Helper as Helper;

class CustomFieldAdmin extends RestAdminController
{

    public function customfield()
    {

        $this->checkPlugin();

        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            $this->listCustomFields($this->request);
        } else {
            $this->statusCode = 405;
            $this->allowedHeaders = array("GET");
        }
        return $this->sendResponse();
    }

    public function listCustomFields($request)
    {

        $this->load->language('extension/restadmin/restadmin/attribute');
        $this->load->model('extension/restadmin/rest/restadmin');

        $parameters = array(
            "limit" => $this->config->get('config_limit_admin'),
            "start" => 1,
        );

        /*check limit parameter*/
        if (isset($request->get['limit']) && $this->isInteger($request->get['limit'])) {
            $parameters["limit"] = $request->get['limit'];
        }

        /*check page parameter*/
        if (isset($request->get['page']) && $this->isInteger($request->get['page'])) {
            $parameters["start"] = $request->get['page'];
        }

        /*group parameter*/
        if (isset($request->get['group']) && $this->isInteger($request->get['group'])) {
            $parameters["filter_attribute_group_id"] = $request->get['group'];
        }

        $parameters["start"] = intval($parameters["start"] - 1) * intval($parameters["limit"]);

        $customfields = array();

        $results = $this->model_extension_restadmin_rest_restadmin->getCustomFields($parameters);

        foreach ($results as $result) {
            $languageId = isset($result['language_id']) ? $result['language_id'] : (int)$this->config->get('config_language_id');

            if($this->multilang){
                $customfields['customfields'][$result['custom_field_id']][] = array(
                    'custom_field_id' => (int)$result['custom_field_id'],
                    'name' => $result['name'],
                    'value' => $result['value'],
                    'type' => $result['type'],
                    'location' => $result['location'],
                    'status' => (int)$result['status'],
                    'sort_order' => (int)$result['sort_order'],
                    'language_id' => $languageId,
                    'values' => $this->model_extension_restadmin_rest_restadmin->getCustomFieldValues($result['custom_field_id'], $languageId)
                );
            } else {
                $customfields['customfields'][$result['custom_field_id']][] = array(
                    'custom_field_id' => (int)$result['custom_field_id'],
                    'name' => $result['name'],
                    'value' => $result['value'],
                    'type' => $result['type'],
                    'location' => $result['location'],
                    'status' => (int)$result['status'],
                    'sort_order' => (int)$result['sort_order'],
                    'language_id' => $languageId,
                    'values' => $this->model_extension_restadmin_rest_restadmin->getCustomFieldValues($result['custom_field_id'], $languageId)
                );
            }
        }

        $this->json['data'] = !empty($customfields) ? $customfields['customfields'] : array();

    }
}