<?php
/**
 * shipping_method_admin.php
 *
 * Shipping method management
 *
 * @author          Opencart-api.com
 * @copyright       2022
 * @license         License.txt
 * @version         1.0
 * @link            https://opencart-api.com/product/opencart-rest-admin-api/
 * @documentations  https://opencart-api.com/opencart-rest-api-documentations/
 */
namespace Opencart\Catalog\Controller\Extension\Restadmin\Rest;

use \Opencart\Extension\Restadmin\System\Engine\RestAdminController;
use \Opencart\System\Helper as Helper;

class ShippingMethodAdmin extends RestAdminController
{

    public function shippingmethods()
    {

        $this->checkPlugin();

        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            $this->listShippingMethods();
        } else {
            $this->statusCode = 405;
            $this->allowedHeaders = array("GET");
        }

        return $this->sendResponse();
    }


    public function listShippingMethods()
    {

        $this->load->model('extension/restadmin/rest/restadmin');
        $this->load->language('extension/shipping');

        $results = $this->model_extension_restadmin_rest_restadmin->getExtensionPaths('%/admin/controller/shipping/%.php');

        foreach ($results as $result) {
            $available[] = basename($result['path'], '.php');
        }

        $installed = [];

        $extensions = $this->model_extension_restadmin_rest_restadmin->getExtensionsByType('shipping');

        foreach ($extensions as $extension) {
            if (in_array($extension['code'], $available)) {
                $installed[] = $extension['code'];
            }
        }

        if ($results) {
            foreach ($results as $result) {
                $extension = substr($result['path'], 0, strpos($result['path'], '/'));

                $code = basename($result['path'], '.php');

                $this->load->language('extension/' . $extension . '/payment/' . $code, $code);

                $text_link = $this->language->get($code . '_text_' . $code);

                if ($text_link != $code . '_text_' . $code) {
                    $link = $text_link;
                } else {
                    $link = '';
                }

                $data['extensions'][] = [
                    'name'       => $this->language->get($code . '_heading_title'),
                    'link'       => $link,
                    'status'     => $this->config->get('payment_' . $code . '_status') ? $this->language->get('text_enabled') : $this->language->get('text_disabled'),
                    'sort_order' => $this->config->get('payment_' . $code . '_sort_order'),
                    'installed'  => in_array($code, $installed),
                ];
            }
        }

        $this->json['data'] = !empty($data['extensions']) ? $data['extensions'] : array();

    }
}
